/**
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.deephacks.tools4j.config.admin;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import javax.jws.WebService;

import org.deephacks.tools4j.config.model.AbortRuntimeException;
import org.deephacks.tools4j.config.model.Bean;
import org.deephacks.tools4j.config.model.Lookup;
import org.deephacks.tools4j.config.spi.Conversion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@WebService(endpointInterface = "org.deephacks.tools4j.config.admin.ConfigAdminPortType",
        portName = "ConfigAdminPort", name = "ConfigAdminService",
        serviceName = "ConfigAdminService",
        targetNamespace = "http://tools4j.deephacks.org/config-service/v1_0",
        wsdlLocation = "WEB-INF/wsdl/ConfigAdminService.wsdl")
public final class ConfigAdminEndpoint implements ConfigAdminPortType {
    Conversion converter = Conversion.get();
    private Logger logger = LoggerFactory.getLogger(ConfigAdminEndpoint.class);

    static {
        Conversion.get().register(new BeanToWsBeanConverter());
        Conversion.get().register(new WsBeanToBeanConverter());
    }

    @Override
    public void create(Beans beans) throws ConfigFault {
        AdminContext admin = Lookup.get().lookup(AdminContext.class);
        Collection<Bean> createbeans = converter.convert(beans.getBeans(), Bean.class);
        try {
            admin.create(createbeans);
        } catch (AbortRuntimeException e) {
            throw abortConfigFault(e);
        } catch (Throwable e) {
            throw throwableConfigFault(e);
        }
    }

    @Override
    public void merge(Beans beans) throws ConfigFault {

        AdminContext admin = Lookup.get().lookup(AdminContext.class);
        Collection<Bean> mergebeans = converter.convert(beans.getBeans(), Bean.class);
        try {
            admin.merge(mergebeans);
        } catch (AbortRuntimeException e) {
            throw abortConfigFault(e);
        } catch (Throwable e) {
            throw throwableConfigFault(e);
        }
    }

    @Override
    public void set(Beans beans) throws ConfigFault {
        AdminContext admin = Lookup.get().lookup(AdminContext.class);
        Collection<Bean> updatebeans = converter.convert(beans.getBeans(), Bean.class);
        try {
            admin.set(updatebeans);
        } catch (AbortRuntimeException e) {
            throw abortConfigFault(e);
        } catch (Throwable e) {
            throw throwableConfigFault(e);
        }

    }

    @Override
    public Beans get(String schemaName, String[] instanceIds) throws ConfigFault {
        AdminContext admin = Lookup.get().lookup(AdminContext.class);
        List<Bean> beans;
        try {
            beans = admin.list(schemaName, Arrays.asList(instanceIds));
            Collection<WsBean> wsbeans = converter.convert(beans, WsBean.class);
            Beans b = new Beans();
            b.getBeans().addAll(wsbeans);
            return b;
        } catch (AbortRuntimeException e) {
            throw abortConfigFault(e);
        } catch (Throwable e) {
            throw throwableConfigFault(e);
        }
    }

    @Override
    public Beans list(String schema) throws ConfigFault {
        AdminContext admin = Lookup.get().lookup(AdminContext.class);
        try {
            List<Bean> beans = admin.list(schema);
            Collection<WsBean> wsBeans = converter.convert(beans, WsBean.class);
            Beans result = new Beans();
            result.getBeans().addAll(wsBeans);
            return result;
        } catch (AbortRuntimeException e) {
            throw abortConfigFault(e);
        } catch (Throwable e) {
            throw throwableConfigFault(e);
        }
    }

    @Override
    public void delete(String schemaName, String[] instanceId) throws ConfigFault {
        AdminContext admin = Lookup.get().lookup(AdminContext.class);
        try {
            admin.delete(schemaName, Arrays.asList(instanceId));
        } catch (AbortRuntimeException e) {
            throw abortConfigFault(e);
        }
    }

    private ConfigFault abortConfigFault(AbortRuntimeException e) {
        ConfigFaultCause cause = new ConfigFaultCause();
        cause.setCode(e.getEvent().getCode());
        cause.setModule(e.getEvent().getModule());
        logger.debug("AbortRuntimeException", e);
        return new ConfigFault(e.getEvent().getMessage(), cause);
    }

    private ConfigFault throwableConfigFault(Throwable e) {
        ConfigFaultCause cause = new ConfigFaultCause();
        cause.setCode(0);
        cause.setModule("unknown");
        logger.warn("Unexpected failure occured: " + e.getMessage());
        logger.debug(e.getClass().getName(), e);
        return new ConfigFault(e.getMessage(), cause);
    }

}
